from casadi import DMatrix, SXMatrix, reshape, horzcat,vertcat,mul,SXFunction
import numpy as NP
import multiprocessing

from iceConcentrationSimulation1D import iceConcentartionSimulation
from iceConcentrationMHE import iceConcentrationMHE
from iceConcentrationMHEmultipleShooting_RK4 import iceConcentrationMHEmultipleShooting
from iceConcentrationPlotting1D import iceConcentrationPlotting, iceConcentrationPlottingComp
from iceConcentrationModel1D import iceConcentrationModel

from iceConcentrationModel2D import iceConcentrationModel as iceConcentrationModel2D
from iceConcentrationPlotting2D import iceConcentrationInteriorPlotting as iceConcentrationInteriorPlotting2D
from iceConcentrationPlotting2D import iceConcentrationPlotting as iceConcentrationPlotting2D
from iceConcentrationParameterPlotting import iceConcentrationParameterPlotting, iceConcentration2ParameterPlotting

from iceConcentrationHEKF import iceConcentrationHEKF
from estimationErrorFcn import estimationErrorFcn


# Distance and number of discretization points in case of 2D
x_L = 0.0; x_R = 750.0 # Distance in x direction
y_L = 0.0; y_R = 750.0 # Distance in y direction   

r = 5 # x-dir
c = 1 # y-dir - Set to 1 for 1D calculation

# Simulation times
t0 = 0.0; # Start time
tf = 900.0; # End time
tn = 21; # Number of discretization points

# MMHE Horizon
nk = 5

# Known parameters
if c == 1:
    p_known = []
else:
    p_known = NP.hstack([NP.ones(c),NP.zeros(c),NP.zeros(r),NP.ones(r)])

# Arrival cost (0 = disable, 1 = enable)
EnableEKFarrivalCost = 1

# Estimation scheme
# 0 = Only HEKF
# 1 = MHE (collocation) + HEKF
# 2 = MHE (Collocation + Multiple shooting) + HEKF
# 3 = MHE (Multiple Shooting) + HEKF
estScheme = 2

# Scenario
# 0 = Constant boundary and velocity
# 1 = Variable boundary, constant velocity
# 2 = Variable boundary, variable velocity
scenario = 0


# The states which are measured
measuredStates = DMatrix.ones(r,c)*1
#measuredStates[0,1] = 1
#measuredStates[2,3] = 1
#measuredStates[1,0] = 1
#measuredStates[3,1] = 1
#measuredStates[2,2] = 1
#measuredStates[1,3] = 1
#measuredStates[0,4] = 1
#measuredStates[2,1] = 1
#measuredStates[1,2] = 1
#measuredStates[0,0] = 1
#measuredStates[0,0] = 1
#measuredStates[4,0] = 1
#measuredStates[5,0] = 1
#measuredStates[9,0] = 1
measuredStatesFlat = reshape(measuredStates,r*c,1)


# Covariance estimates
CovKnownStates = 0.1
CovUnknownStates = 1.1
CovUnknownParameters = 1.5
CovKnownStatesMeasured = 0.1

# MHE weighting matrix
Vj = DMatrix.eye(r*c)*0 
for i in range(r*c):
    Vj[i,i] = measuredStatesFlat[i]



# Number of measured states
ny = 0
for i in measuredStatesFlat:
    if i==1:
        ny +=1


if __name__=="__main__":
    
    print "Setting up model"
    if c==1:
        f_rhs, c_bar, p, dot_c = iceConcentrationModel(x_R,x_L,r)
    else:
        f_rhs, c_bar, p, dot_c = iceConcentrationModel2D(x_R,x_L,y_R,y_L,r,c)
    print "Setting up model finished"
    
    #Number of unknown parameters
    np = p.shape[0]-len(p_known)
    
    # Hybrid EKF parameters x_dot = f(x,u,w,t), y = h_k(x_k,v_k)
    M_k = DMatrix.eye(ny) # Partial derivative of h_k with respect to v_k - Chosen to be the identity matrix since we assume the same noise on every state
    H_k = DMatrix.zeros(ny,r*c+np) # Partial derivative of h_k with respect to x_k
    j = 0
    for i in range(r*c):
        if measuredStatesFlat[i]==1:
            H_k[j,i]=1
            j+=1
            
    # Covariance matrices
    R = DMatrix.eye(ny)*CovKnownStatesMeasured
    Q = DMatrix.eye(r*c+np)*CovUnknownStates
    for i in range(r*c):
        if measuredStatesFlat[i] == 1:
            Q[i,i] = CovKnownStates
    for i in range(np):
        Q[r*c+i,r*c+i] = CovUnknownParameters
        
    # Calculate linearized observability:
    f_rhs.init()
    dfdx = f_rhs.jac(0)
    dfdp = f_rhs.jac(2)[:,0:np]
    A1 = horzcat([dfdx, dfdp])
    zeroMat = NP.zeros((np, r*c+np))
    A = vertcat([A1, zeroMat])
    C = H_k
    ObsMat = SXMatrix(ny*(r*c+np),r*c+np)
    for i in range(r*c+np):
        if i == 0:
            ObsMat[i*(ny):i*(ny)+ny,:]=C
        else:
            A_exp = NP.eye(r*c+np)
            for j in range(i):
                A_exp = mul(A_exp,A)
            ObsMat[i*(ny):i*(ny)+ny,:]=mul(C,A_exp)
    ObsMatFcn = SXFunction([c_bar,p],[ObsMat])
    ObsMatFcn.init()
    test = [DMatrix.zeros(r*c,1),DMatrix.ones(np,1)]
    ObsMatFcn.setInput(DMatrix.ones(r*c,1)*0,0)
    ObsMatFcn.setInput(vertcat([DMatrix.ones(np,1),p_known]),1)
    ObsMatFcn.evaluate()
    print "Rank of oberservability matrix",NP.linalg.matrix_rank(ObsMatFcn.output())
    
    knownParametersVector = NP.zeros((tn,len(p_known)))
    for i in range(tn):
        knownParametersVector[i,:]=p_known
            
    #################################################################
    ############## Run the simulation ###############################
    #################################################################
    
    print "Running simulation"
    iceConcentartionSimulation(f_rhs,r,c,t0,tf,tn,scenario)
    print "Simulation finished"

    
    # Read data from file and plot
    simulationData = NP.genfromtxt('simulationDataWithNoise.txt', delimiter=',')
    simulationParameters = NP.genfromtxt('simulationParameters.txt', delimiter=',')
    if c == 1:
        simulationPlotProcess = multiprocessing.Process(target = iceConcentrationPlotting, args = (x_R,x_L,r,t0,tf,tn,simulationData,"Simulation",))
    else:
        simulationPlotProcess = multiprocessing.Process(target = iceConcentrationInteriorPlotting2D, args = (x_R,x_L,y_R,y_L,r,c,t0,tf,tn,simulationData,"Simulation",))
    simulationPlotProcess.daemon = True
    simulationPlotProcess.start()
    


    #################################################################
    ############## Choose estimation scheme #########################
    #################################################################
    if estScheme == 1 or estScheme == 2: # MHE with direct collocation + HEKF:
        print "Running estimation (Collocation)"
        iceConcentrationMHE(f_rhs, c_bar, p, dot_c ,simulationData, t0, tf, tn, nk, p_known, np, Vj, ny, M_k, H_k,Q,R,EnableEKFarrivalCost,ObsMatFcn)
        print "estimation finished"

        # Read data from file and plot
        estimationData = NP.genfromtxt('MHEdata.txt', delimiter=',')
        estimationParameters = NP.genfromtxt('MHE_param_data.txt', delimiter=',')
        estimationDataKF = NP.genfromtxt('HEKF_estimation_data.txt', delimiter=',')
        estimationParametersKF = estimationDataKF[:,-np:]
        estimationStatesKF = estimationDataKF[:,:r*c]
        estimationError = simulationData[nk:tn+1,:]-estimationData
        if np == 0:
            estimationParameters = NP.zeros((tn-nk,np))
        elif np == 1:
            estimationParameters=NP.array([estimationParameters]).T
        estimationErrorParam = NP.abs(simulationParameters[nk:tn+1,:]-NP.hstack([estimationParameters,knownParametersVector[nk:tn+1,:]]))
    
    if estScheme == 2 or estScheme == 3: # MHE with multiple shooting + HEKF:
        print "Running estimation (Multiple Shooting)"
        iceConcentrationMHEmultipleShooting(f_rhs, c_bar, p, dot_c ,simulationData, t0, tf, tn, nk, p_known, np, Vj, ny, M_k, H_k,Q,R,EnableEKFarrivalCost,ObsMatFcn)
        print "estimation finished"    
        estimationDataRK4 = NP.genfromtxt('MHEstateEstimateRK4.txt', delimiter=',')
        estimationParametersRK4 = NP.genfromtxt('MHEparamEstimateRK4.txt', delimiter=',')
        if np == 0:
            estimationParametersRK4 = NP.zeros((tn-nk,np))
        elif np == 1:
            estimationParametersRK4=NP.array([estimationParametersRK4]).T
        
        if estScheme == 3:
            estimationDataKF = NP.genfromtxt('HEKFestimationData_multipleShooting.txt', delimiter=',')
            estimationParametersKF = estimationDataKF[:,-np:]
            estimationStatesKF = estimationDataKF[:,:r*c]
            estimationError = simulationData[nk:tn+1,:]-estimationDataRK4
            estimationErrorParam = NP.abs(simulationParameters[nk:tn+1,:]-NP.hstack([estimationParametersRK4,knownParametersVector[nk:tn+1,:]]))
            estimationData = estimationDataRK4
            estimationParameters = estimationParametersRK4
            
    if estScheme == 0: # HEKF
        print "Running estimation (HEKF)"
        iceConcentrationHEKF(f_rhs, c_bar, p, dot_c,simulationData,t0,tf,tn,p_known,np,ny,M_k,H_k,Q,R,ObsMatFcn)
        print "estimation finished"
        estimationDataKF = NP.genfromtxt('HEKFestimationData.txt', delimiter=',')
        estimationParametersKF = estimationDataKF[:,-np:]
        estimationStatesKF = estimationDataKF[:,:r*c]
        estimationError = simulationData-estimationStatesKF
        estimationErrorParam = NP.abs(simulationParameters-NP.hstack([estimationParametersKF,knownParametersVector]))
        estimationData = estimationStatesKF
        estimationParameters = estimationParametersKF

    #################################################################
    ############## Plotting the results #############################
    #################################################################
    if c == 1:
        estimationPlotProcess = multiprocessing.Process(target = iceConcentrationPlotting, args = (x_R,x_L,r,t0,tf,tn,estimationData,"Estimation",))
    else:
        estimationPlotProcess = multiprocessing.Process(target = iceConcentrationInteriorPlotting2D, args = (x_R,x_L,y_R,y_L,r,c,t0,tf,tn,estimationData,"Estimation",))
    estimationPlotProcess.daemon = True
    estimationPlotProcess.start()
    
    

        
    
    
    if c == 1:
        estimationErrorPlotProcess = multiprocessing.Process(target = iceConcentrationPlotting, args = (x_R,x_L,r,t0,tf,tn,estimationError,"Estimation error",))
        if estScheme != 0:
            estimationPlotCompProcess = multiprocessing.Process(target = iceConcentrationPlottingComp, args = (x_R,x_L,r,t0,tf,tn,simulationData,estimationStatesKF,estimationData,"Real (simulated) and estimated ice concentration",["Real","EKF","MHE"],))
            estimationPlotCompProcess.daemon = True
            estimationPlotCompProcess.start()
    else:
        estimationErrorPlotProcess = multiprocessing.Process(target = iceConcentrationInteriorPlotting2D, args = (x_R,x_L,y_R,y_L,r,c,t0,tf,tn,estimationError,"Estimation error",))
    estimationErrorPlotProcess.daemon = True
    estimationErrorPlotProcess.start()
    
    
    if estScheme == 0:
        print "EKF error:", (estimationErrorFcn(estimationStatesKF,estimationParametersKF,simulationData,simulationParameters))
    if estScheme == 1 or estScheme == 2:
        print "MHE - collocation error:", (estimationErrorFcn(estimationData,estimationParameters,simulationData,simulationParameters))
        print "EKF error:", (estimationErrorFcn(estimationStatesKF[nk:,:],estimationParametersKF[nk:,:],simulationData,simulationParameters))
    if estScheme == 2 or estScheme == 3:
        print "MHE - shooting error:", (estimationErrorFcn(estimationDataRK4,estimationParametersRK4,simulationData,simulationParameters))
        print "EKF error:", (estimationErrorFcn(estimationStatesKF[nk:,:],estimationParametersKF[nk:,:],simulationData,simulationParameters))

    if estScheme != 0:
        if c == 1:
            if np == 1:
                iceConcentrationParameterPlotting(t0,tf,tn,np,estimationParameters,estimationParametersKF,simulationParameters,["Velocity estimation"],['Real','EKF', 'MHE'],["$u$ [m/s]"])
            elif np == 2:
                iceConcentrationParameterPlotting(t0,tf,tn,np,estimationParameters,estimationParametersKF,simulationParameters,["Velocity estimation","Left concentration estimation"],['Real','EKF', 'MHE'],["$u$ [m/s]","$c$ [-]"])
            elif np == 3:
                iceConcentrationParameterPlotting(t0,tf,tn,np,estimationParameters,estimationParametersKF,simulationParameters,["Velocity estimation","Left concentration estimation","Right concentration estimation"],['Real','EKF', 'MHE'],['$u$ [m/s]','$c$ [-]','$c$ [-]'])
    if c >1:
        iceConcentration2ParameterPlotting(t0,tf,tn,np,simulationParameters,estimationParametersKF[:,0:2],["Velocity estimation (u)","Velocity estimation (v)"],['Real','EKF'],["$u$ [m/s]","$v$ [m/s]"])

    estimationPlotProcess.join()
    estimationErrorPlotProcess.join() 
    simulationPlotProcess.join()
    print 'Thread terminated'
