## The advection equation discretized and implemented by Joakim Haugen
## Modified in order to get a uniform u

from numpy import *
import numpy
from casadi import *
from pylab import *

def iceConcentrationModel(x_R,x_L,r):
    Dx = (x_R-x_L)/r
    
    c_bar = ssym("c",r,1)
    con_L = ssym("c_Left")
    con_R = ssym("c_Right")
    bound = [con_L, con_R]
    
    
    u = ssym("u",1)
    dot_c = SXMatrix(r,1)
    
    
    c_L = 1.0/4
    c_R = 1.0/4
    c_C = 1.0/2
    eps = 1e-6
    
    def cm1p1(j,c_bar,bound):
        if j==0:
            cm1 = bound[0]
            c = bound[0]
            cp1 = c_bar[j]
        elif j==1:
            cm1 = bound[0]
            c = c_bar[j-1]
            cp1 = c_bar[j]
        elif j<c_bar.size1():
            cm1 = c_bar[j-2]
            c = c_bar[j-1]
            cp1 = c_bar[j]
        elif j==c_bar.size1():
            cm1 = c_bar[j-2]
            c = c_bar[j-1]
            cp1 = bound[1]
        elif j==c_bar.size1()+1:
            cm1 = c_bar[j-2]
            c = bound[1]
            cp1 = bound[1]
        return [cm1,c,cp1]
    
    def um1p1(j,u):
        if j==0:
            um1 = u[0]
            un = u[0]
            up1 = u[0]
        elif j==1:
            um1 = u[0]
            un = u[0]
            up1 = u[0]
        elif j<u.size1():
            um1 = u[0]
            un = u[0]
            up1 = u[0]
        elif j==u.size1():
            um1 = u[0]
            un = u[0]
            up1 = u[0]
        elif j==u.size1()+1:
            um1 = u[0]
            un = u[0]
            up1 = u[0]
        return [um1,un,up1]
    
    def IS_L(j,c_bar,bound):
        cs = cm1p1(j,c_bar,bound)
        return (cs[1] - cs[0])**2
    
    def IS_R(j,c_bar,bound):
        cs = cm1p1(j,c_bar,bound)
        return (cs[2] - cs[1])**2
    
    def IS_C(j,c_bar,bound):
        cs = cm1p1(j,c_bar,bound)
        return (13.0*(cs[2] - 2*cs[1] + cs[0])**2)/3.0 + ((cs[2]-cs[0])**2)/4.0
    
    def alpha_L(j,c_bar,bound):
        return c_L/(eps+IS_L(j,c_bar,bound))**2
    
    def alpha_R(j,c_bar,bound):
        return c_R/(eps+IS_R(j,c_bar,bound))**2
    
    def alpha_C(j,c_bar,bound):
        return c_C/(eps+IS_C(j,c_bar,bound))**2
    
    def sum_alpha(j,c_bar,bound):
        return alpha_C(j,c_bar,bound) + alpha_L(j,c_bar,bound) + alpha_R(j,c_bar,bound)
    
    def w_L(j,c_bar,bound):
        return alpha_L(j,c_bar,bound)/sum_alpha(j,c_bar,bound)
    
    def w_R(j,c_bar,bound):
        return alpha_R(j,c_bar,bound)/sum_alpha(j,c_bar,bound)
    
    def w_C(j,c_bar,bound):
        return alpha_C(j,c_bar,bound)/sum_alpha(j,c_bar,bound)
    
    def Aj(j,c_bar,bound):
        cs = cm1p1(j,c_bar,bound)
        return cs[1] - w_C(j,c_bar,bound)*(cs[2] - 2*cs[1] + cs[0])/12.0
    
    def Bj(j,c_bar,bound):
        cs = cm1p1(j,c_bar,bound)
        return (w_R(j,c_bar,bound)*(cs[2]-cs[1])+w_C(j,c_bar,bound)*(cs[2]-cs[0])/2.0+w_L(j,c_bar,bound)*(cs[1]-cs[0]))/Dx
    
    def Cj(j,c_bar,bound):
        cs = cm1p1(j,c_bar,bound)
        return 2*w_C(j,c_bar,bound)*(cs[0]-2*cs[1] + cs[2])/(Dx**2)
    
    def cplus(j,c_bar,bound):
        return Aj(j+1,c_bar,bound) - Dx*Bj(j+1,c_bar,bound)/2.0 + (Dx**2/8.0)*Cj(j+1,c_bar,bound)
    
    def cminus(j,c_bar,bound):
        return Aj(j,c_bar,bound) + Dx*Bj(j,c_bar,bound)/2.0 + (Dx**2/8.0)*Cj(j,c_bar,bound)
    
    def aj(j,u):
        #us = um1p1(j,u)
        return fabs(u[0])
    
    def f(j,c,u):
        #us = um1p1(j,u)
        return u[0]*c
    
    
    for j in range(1,r+1):
    #    print j
        dot_c[j-1,0] = - (f(j,cplus(j,c_bar,bound),u) + f(j,cminus(j,c_bar,bound),u))/(2*Dx) \
        + (f(j-1,cplus(j-1,c_bar,bound),u) + f(j-1,cminus(j-1,c_bar,bound),u))/(2*Dx) \
        + aj(j,u)*(cplus(j,c_bar,bound)- cminus(j,c_bar,bound))/(2*Dx) \
        - aj(j-1,u)*(cplus(j-1,c_bar,bound)-cminus(j-1,c_bar,bound))/(2*Dx)
 
    p = vertcat([u,con_L,con_R])
    f_rhs = SXFunction(daeIn(x=c_bar,p=p),daeOut(ode=dot_c))
    
    return f_rhs, c_bar, p, dot_c