import numpy as NP
import pylab
import matplotlib.pyplot as plt
import matplotlib.animation as animation

def iceConcentrationPlotting(x_R,x_L,r,t0,tf,tn,sol,name):    
    fig = plt.figure()
    ax = fig.add_subplot(111)
    line, = ax.plot([], [], lw=2)
    ts = sol.shape[0]
    ax.set_ylim(0, 1.1*NP.max(sol))
    ax.set_xlim(x_L, x_R)
    dx = (x_R-x_L)/(r-1)
    
    ax.axvline(x=0,linewidth=1,color='k',ls=':')
    for i in range(r):
        ax.axvline(x=dx*(i+1),linewidth=1,color='k',ls=':')
    ax.yaxis.grid(True)
    
    xdata, ydata = [], []
    
#    rc('text', usetex=True)
    pylab.rc('font', family='serif')
    plt.xlabel(r'$x$ [m]')
    plt.ylabel(r'$\bar c$ [-]')
    plt.title(name+", time: "+str(0)+"s",fontsize=16)
    
    def init():
        line.set_data([],[])
        return line,
    
    def animate(i):
        xdata= NP.linspace(x_L,x_R,r)
        ydata= sol[i,:]
        line.set_data(xdata, ydata)
        plt.title(name+", time: "+str((i+tn-ts)*((tf-t0)/(tn-1)))+"s",fontsize=16)
        if i % 10 == 0:
            pylab.savefig('plots/'+name+'_'+str(i)+'.pdf', bbox_inches=0)
        return line,
    
    ani = animation.FuncAnimation(fig, animate, init_func=init, frames=ts,interval=((tf-t0)/tn)*10, repeat_delay=2000, repeat=True)
    plt.show()
    
    
    
def iceConcentrationPlottingComp(x_R,x_L,r,t0,tf,tn,real_sol,est1_sol,est2_sol,name,legend):    
    fig = plt.figure()
    ax = fig.add_subplot(111)
    line, = ax.plot([], [], lw=2, c= 'c')
    line2, = ax.plot([], [], lw=2, ls = '-.',c='r')
    line3, = ax.plot([], [], lw=2, ls = '--',c='b')
    
#    ts = NP.linspace(t0,tf,tn)
    ts = NP.max([real_sol.shape[0],est1_sol.shape[0],est2_sol.shape[0]])
    ts1 = est1_sol.shape[0]
    ts2 = est1_sol.shape[0]
    ts3 = est2_sol.shape[0]
    ts_diff1 = ts - ts1
    ts_diff2 = ts - ts2
    ts_diff3 = ts - ts3
    ax.set_ylim(0, 1.1*NP.max(real_sol))
    ax.set_xlim(x_L, x_R)
#    ax.grid()
    dx = (x_R-x_L)/(r-1)
    
    ax.axvline(x=0,linewidth=1,color='k',ls=':')
    for i in range(r):
        ax.axvline(x=dx*(i+1),linewidth=1,color='k',ls=':')
    ax.yaxis.grid(True)
    
    xdata, ydata = [], []
    
#    rc('text', usetex=True)
    pylab.rc('font', family='serif')
    plt.xlabel(r'$x$ [m]')
    plt.ylabel(r'$\bar c$ [-]')
    plt.title(name+", time: "+str(0)+'s',fontsize=16)
    
    def init():
        line.set_data([],[])
        line2.set_data([],[])
        line3.set_data([],[])
        return line,line2,line3
    
    def animate(i):
        xdata= NP.linspace(x_L,x_R,r)
        ydata= real_sol[i,:]
        
        if(i >= ts_diff3):
            ydata2 = est2_sol[i-ts_diff3,:]
            xdata2 = NP.linspace(x_L,x_R,r)
        else:
            ydata2 = []
            xdata2 = []
        if(i >= ts_diff2):
            ydata1 = est1_sol[i-ts_diff2,:]
            xdata1 = NP.linspace(x_L,x_R,r)
        else:
            ydata1 = []
            xdata1 = []
        line.set_data(xdata, ydata)
        line2.set_data(xdata1, ydata1)
        line3.set_data(xdata2, ydata2)
        plt.title(name+", time: "+str((i+tn-ts)*((tf-t0)/(tn-1)))+"s",fontsize=16)
        plt.legend(legend, loc='best')
        if i % 10 == 0:
            pylab.savefig('plots/'+name+'_comp_'+str(i)+'.pdf', bbox_inches=0)
        elif i == ts-1:
            pylab.savefig('plots/'+name+'_comp_'+str(i)+'.pdf', bbox_inches=0)
        return line,line2
    
    ani = animation.FuncAnimation(fig, animate, init_func=init, frames=ts,interval=((tf-t0)/(tn-1))*10, repeat_delay=2000, repeat=True)
    plt.show()