from numpy import *
import numpy
from casadi import *
from pylab import *
import matplotlib.pyplot as plt
import matplotlib.animation as animation
import time
import types

def iceConcentrationPlotting(x_R,x_L,y_R,y_L,r,c,t0,tf,tn,sol,param,name):
    c_left = DMatrix.zeros(1,c)
    c_right = DMatrix.zeros(1,c)
    c_top = DMatrix.zeros(r,1)
    c_bottom = DMatrix.zeros(r,1)

    c_b_init = vertcat([param[0,2:2*r+2*c+2]])
    c_left_T = c_b_init[0:c]
    c_right_T = c_b_init[c:c*2]
    c_top[0:r,0] = c_b_init[2*c:2*c+r]
    c_bottom[0:r,0] = c_b_init[2*c+r:2*c+2*r]
    c_left[0,0:c] = c_left_T.T
    c_right[0,0:c] = c_right_T.T
    
    
    Dx = abs((x_R-x_L)/r)
    Dy = abs((y_R-y_L)/c)
    
    ts = sol.shape[0]
        
    nolvl = 25
    levels =array(range(nolvl))/(1.0*(nolvl-1))
    levels[0] = -.04
    levels[nolvl-1] = 1.05
    tr = array(range(0,len(levels),1))/(1.0*(len(levels)-1))
    
    color = []
    for i in range(len(levels)):
        color.append((tr[i],tr[i],1.0))
    
    fig1 = plt.figure()
    ax = fig1.add_subplot(111)#,projection='3d')
    Xx = numpy.arange(x_L, x_R+2*Dx, Dx)
    Yy = numpy.arange(y_L, y_R+2*Dy, Dy)
    ax.set_xlim(x_L,x_R+Dx); ax.set_ylim(y_L,y_R+Dy)
    Zz = DMatrix(r+2,c+2); Zz[0,1:c+1] = c_left; Zz[1:r+1,0] = c_bottom; Zz[r+1,1:c+1] = c_right; Zz[1:r+1,c+1] = c_top
    Zz[0,0] = Zz[1,0]*0.5 + Zz[0,1]*0.5
    Zz[r+1,0] = Zz[r,0]*0.5 + Zz[r+1,1]*0.5
    Zz[r+1,c+1] = Zz[r,c+1]*0.5 + Zz[r+1,c]*0.5
    Zz[0,c+1] = Zz[0,c]*0.5 + Zz[1,c+1]*0.5
    Zz[1:r+1,1:c+1] = casadi.reshape(sol[0,:],r,c)
    Z = Zz.toArray()
    Z = trans(Z)
    im = plt.contourf(Xx,Yy,Z,levels,colors=color)
     
    # rc('text', usetex=True)
    rc('font', family='serif')
    plt.xlabel(r'$x$ [m]')
    plt.ylabel(r'$y$ [m]')
    plt.title(name+", time: "+str(0),fontsize=16)
    dx = (x_R-x_L)/(r)
    dy = (y_R-y_L)/(c)
    ax.axvline(x=0,linewidth=1,color='k',ls=':')
    ax.axhline(y=0,linewidth=1,color='k',ls=':')
    for i in range(r+2):
        ax.axvline(x=dx*(i+1),linewidth=1,color='k',ls=':')
    for i in range(c+2):
        ax.axhline(y=dy*(i+1),linewidth=1,color='k',ls=':')
    
     
     
     
     
    ims = []
    def animate(i):
        u_init = param[i,0]
        v_init = param[i,1]
        c_b_init = vertcat([param[i,2:2*r+2*c+2]])
        c_left_T = c_b_init[0:c]
        c_right_T = c_b_init[c:c*2]
        c_top = c_b_init[2*c:2*c+r]
        c_bottom = c_b_init[2*c+r:2*c+2*r]
        c_left = c_left_T.T
        c_right = c_right_T.T
        
        Zz = DMatrix(r+2,c+2); Zz[0,1:c+1] = c_left; Zz[1:r+1,0] = c_bottom; Zz[r+1,1:c+1] = c_right; Zz[1:r+1,c+1] = c_top
        Zz[0,0] = Zz[1,0]*0.5 + Zz[0,1]*0.5
        Zz[r+1,0] = Zz[r,0]*0.5 + Zz[r+1,1]*0.5
        Zz[r+1,c+1] = Zz[r,c+1]*0.5 + Zz[r+1,c]*0.5
        Zz[0,c+1] = Zz[0,c]*0.5 + Zz[1,c+1]*0.5
        Zz[1:r+1,1:c+1] = casadi.reshape(sol[i,:],r,c)
        Z = Zz.toArray()
        Z = trans(Z)
        im = plt.contourf(Xx,Yy,Z,levels,colors=color)
        def setvisible(self,vis):
            for c in self.collections: c.set_visible(vis)
        im.set_visible = types.MethodType(setvisible,im,None)
        im.axes = plt.gca()
        im.figure = fig1
        im.cmap.set_over((1.0,1.0,1.0))
        im.cmap.set_bad((1.0,1.0,1.0))
        im.cmap.set_under((0.0,0.0,0.0))
        plt.title(name+", time: "+str((i+tn-ts)*((tf-t0)/(tn-1))),fontsize=16)
        if i % 10 == 0:
            savefig('plots/'+name+'_'+str(i)+'_2D.pdf', bbox_inches=0)
     
    ani = animation.FuncAnimation(fig1, animate, frames=ts,interval=((tf-t0)/(tn-1))*10, repeat_delay=2000, repeat=True) 
    plt.colorbar(im,ax=ax,shrink=0.8)
    plt.show()
    
    
def iceConcentrationInteriorPlotting(x_R,x_L,y_R,y_L,r,c,t0,tf,tn,sol,name):
    Dx = abs((x_R-x_L)/(r-1))
    Dy = abs((y_R-y_L)/(c-1))
    
    ts = sol.shape[0]
        
    nolvl = 25
    levels =array(range(nolvl))/(1.0*(nolvl-1))
    levels[0] = -.04
    levels[nolvl-1] = 1.05
    tr = array(range(0,len(levels),1))/(1.0*(len(levels)-1))
    
    color = []
    for i in range(len(levels)):
        color.append((tr[i],tr[i],1.0))
    
    fig1 = plt.figure()
    ax = fig1.add_subplot(111)
    Xx = numpy.arange(x_L, x_R+Dx, Dx)
    print "Xx",Xx.shape
    Yy = numpy.arange(y_L, y_R+Dy, Dy)
    print "Yy",Yy.shape
    ax.set_xlim(x_L,x_R); ax.set_ylim(y_L,y_R)
    Zz = DMatrix(r,c); 
    Zz[0:r,0:c] = casadi.reshape(sol[0,:],r,c)
    Z = Zz.toArray()
    Z = trans(Z)
    print "Z",Z.shape
    im = plt.contourf(Xx,Yy,Z,levels,colors=color)
     
    # rc('text', usetex=True)
    rc('font', family='serif')
    plt.xlabel(r'$x$ [m]')
    plt.ylabel(r'$y$ [m]')
    plt.title(name+", time: "+str(0),fontsize=16)
    dx = (x_R-x_L)/(r-1)
    dy = (y_R-y_L)/(c-1)
    ax.axvline(x=0,linewidth=1,color='k',ls=':')
    ax.axhline(y=0,linewidth=1,color='k',ls=':')
    for i in range(r):
        ax.axvline(x=dx*(i),linewidth=1,color='k',ls=':')
    for i in range(c):
        ax.axhline(y=dy*(i),linewidth=1,color='k',ls=':')
    
     
     
     
     
    ims = []
    def animate(i):
        Zz = DMatrix(r,c);
        Zz[0:r,0:c] = casadi.reshape(sol[i,:],r,c)
        Z = Zz.toArray()
        Z = trans(Z)
        im = plt.contourf(Xx,Yy,Z,levels,colors=color)
        def setvisible(self,vis):
            for c in self.collections: c.set_visible(vis)
        im.set_visible = types.MethodType(setvisible,im,None)
        im.axes = plt.gca()
        im.figure = fig1
        im.cmap.set_over((1.0,1.0,1.0))
        im.cmap.set_bad((1.0,1.0,1.0))
        im.cmap.set_under((0.0,0.0,0.0))
        plt.title(name+", time: "+str((i+tn-ts)*((tf-t0)/(tn-1))),fontsize=16)
        if i % 10 == 0:
            savefig('plots/'+name+'_'+str(i)+'_2D.pdf', bbox_inches=0)
    ani = animation.FuncAnimation(fig1, animate, frames=ts,interval=((tf-t0)/(tn-1))*10, repeat_delay=2000, repeat=True) 
    plt.colorbar(im,ax=ax,shrink=0.8)
    plt.show()