import numpy as NP
from casadi import *
import time

def iceConcentartionSimulation(f_rhs,r,c,t0,tf,tn,scenario):
    print "simulation started"
 
    # New variables for noise
    sigma_p = 0.01
    simulated_V = DMatrix(sigma_p*NP.random.randn(tn,r*c))*0 # Measurement noise  
    
    # Initial values
    c_init = 0*DMatrix.ones(r,c)
    c_left = DMatrix.ones(1,c)
    c_left_0 = DMatrix.zeros(1,c)
    c_left_1 = DMatrix.ones(1,c)
    c_right = DMatrix.zeros(1,c)
    
    c_top = DMatrix.ones(r,1)*0
    c_bottom = DMatrix.ones(r,1)*0
    c_bottom_1 = DMatrix.ones(r,1)*1
    c_bottom_0 = DMatrix.ones(r,1)*0

    u_init = DMatrix.ones(1,1)*0.75
    v_init = DMatrix.ones(1,1)*0.5
    
    c_bar_init = casadi.reshape(c_init,r*c,1)
    c_b_init = vertcat([c_left.T,c_right.T,c_top,c_bottom])
    c_b_0 = vertcat([c_left_0.T,c_right.T,c_top,c_bottom_0])
    c_b_1 =vertcat([c_left_1.T,c_right.T,c_top,c_bottom_1])
    
    if c==1:
        p_init = vertcat([u_init,c_left,c_right])
        p_pulse_0 = vertcat([u_init,c_left_0,c_right])
        p_pulse_1 = vertcat([u_init,c_left_1,c_right])
        p_pulse_velocity = vertcat([1,0,0])
    else:
        p_init = vertcat([u_init,v_init,c_b_init])
        p_pulse_0 = vertcat([u_init,v_init,c_b_0])
        p_pulse_1 = vertcat([u_init,v_init,c_b_1])

    
    
    simulated_y = DMatrix.zeros(tn,r*c)
    simulated_p = DMatrix.ones(tn,p_init.shape[0])*1
    
    if scenario == 0: # Constant boundary and speed
        for i in range(tn):#
            if i < tn/50:
                simulated_p[i,:] = p_pulse_1.T
            elif i < (tn*2)/10:
                simulated_p[i,:] = p_pulse_1.T
            elif i < (tn*3)/10:
                simulated_p[i,:] = p_pulse_1.T
            elif i < (tn*4)/10:
                simulated_p[i,:] = p_pulse_1.T
            elif i < (tn*5)/10:
                simulated_p[i,:] = p_pulse_1.T
            else:
                simulated_p[i,:] = p_pulse_1.T        
    elif scenario == 1: # Variable boundary, constant speed
        for i in range(tn):#
            if i < tn/50:
                simulated_p[i,:] = p_pulse_0.T
            elif i < (tn*2)/10:
                simulated_p[i,:] = p_pulse_1.T
            elif i < (tn*3)/10:
                simulated_p[i,:] = p_pulse_1.T
            elif i < (tn*4)/10:
                simulated_p[i,:] = p_pulse_1.T
            elif i < (tn*5)/10:
                simulated_p[i,:] = p_pulse_0.T
            else:
                simulated_p[i,:] = p_pulse_0.T
    elif scenario == 2:
        for i in range(tn): # Variable boundary, variable speed
            if i < tn/50:
                simulated_p[i,:] = p_pulse_1.T-0.012*i*p_pulse_velocity.T
            elif i < (tn*2)/10:
                simulated_p[i,:] = p_pulse_1.T-0.012*i*p_pulse_velocity.T
            elif i < (tn*3)/10:
                simulated_p[i,:] = p_pulse_1.T-0.012*i*p_pulse_velocity.T
            elif i < (tn*4)/10:
                simulated_p[i,:] = p_pulse_1.T-0.012*i*p_pulse_velocity.T
            elif i < (tn*5)/10:
                simulated_p[i,:] = p_pulse_0.T-0.012*i*p_pulse_velocity.T
            else:
                simulated_p[i,:] = p_pulse_0.T-0.012*i*p_pulse_velocity.T
            

    integrator = CVodesIntegrator(f_rhs)
    integrator.setOption("abstol",1e-6)
    integrator.setOption("reltol",1e-6)
    integrator.setOption("t0",t0)
    integrator.setOption("tf", (tf)/(tn-1))
    integrator.init()
    
    tic = time.time()
    simulated_y[0,:] = c_bar_init.T
    for i in range(tn-1):
        integrator.input(INTEGRATOR_X0).set(simulated_y[i,:].T)
        integrator.input(INTEGRATOR_P).set(simulated_p[i,:].T)
        integrator.evaluate()
        simulated_y[i+1,:] = integrator.output().T
    NP.savetxt("simulationDataWithoutNoise.txt",simulated_y, delimiter = ',')
    simulated_y += simulated_V
    
    
    print "Time spent on simulation:",time.time() - tic
    NP.savetxt("simulationDataWithNoise.txt",simulated_y, delimiter = ',')
    NP.savetxt("simulationParameters.txt",simulated_p, delimiter = ',')
    return